# Generated by Django 4.0.10 on 2024-05-21 12:29
from contextlib import closing
from django.db import connections
from django.db import migrations
from django.db import models
from django.db import router
from django_postgres_partitioning import get_model_partitions
import django.db.models.deletion


def create_partitions_message_id_unique_indexes(apps, schema_editor):
    model = apps.get_model('adapter_client', 'IncomingMessage')
    partitions = get_model_partitions(model)
    sql_tmpl = """
        DROP INDEX public.{0}_message_id_idx;
        CREATE UNIQUE INDEX {0}_message_id_idx ON public.{0} (message_id);
    """

    database_alias = router.db_for_write(model)
    with closing(connections[database_alias].cursor()) as cursor:
        for partition_name in partitions:
            cursor.execute(sql_tmpl.format(partition_name))


class Migration(migrations.Migration):

    dependencies = [
        ('adapter_client', '0003_message_transaction_code_alter_config_journal_config_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='incomingmessage',
            name='message',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, to='adapter_client.message'),
        ),
        migrations.RunPython(create_partitions_message_id_unique_indexes, migrations.RunPython.noop)
    ]
